==> pygpiotools-0.1.0.tar.gz: pygpiotools-0.1.0/pygpiotools/pl2303.py <==
import pygpiotools
import usb.core
import usb.util


VID = 0x67b
PID = 0x2303

class gpio:
    def __init__(self, dir_reg, dir_mask, out_reg, out_mask):
        self.dir_reg = dir_reg
        self.dir_mask = dir_mask
        self.out_reg = out_reg
        self.out_mask = out_mask

# Tested with good old PL2303HXA
__gpios = [ gpio(0x01, 0x10, 0x01, 0x40), gpio(0x01, 0x20, 0x01, 0x80)]

def pin_to_gpio(pin):
    if pin > len(__gpios):
        raise Exception(f"invalid gpio number {pin}")
    return __gpios[pin]

def open(driver, device):
    handle = pygpiotools.tty_to_usbdev(device, VID, PID)
    return handle

def __read_pl2303_reg(device, reg_num):
    buffer = device.ctrl_transfer(
        usb.util.build_request_type(usb.util.CTRL_IN, usb.util.CTRL_TYPE_VENDOR, usb.util.CTRL_RECIPIENT_DEVICE),
        0x01,
        reg_num | 0x80,
        0,
        1,
        1000)
    return buffer[0]

def __write_pl2303_reg(device, reg_num, value):
    device.ctrl_transfer(
        usb.util.build_request_type(usb.util.CTRL_OUT, usb.util.CTRL_TYPE_VENDOR, usb.util.CTRL_RECIPIENT_DEVICE),
        0x01,
        reg_num,
        value,
        None,
        1000)

def set_gpio_out(device, gpio, v):
    dir_reg_value = __read_pl2303_reg(device, gpio.dir_reg)
    dir_reg_value |= gpio.dir_mask
    __write_pl2303_reg(gpio.dir_reg, dir_reg_value)

    out_reg_value = __read_pl2303_reg(device, gpio.out_reg)
    if v:
        out_reg_value |= gpio.out_mask
    else:
        out_reg_value &= ~gpio.out_mask
    __write_pl2303_reg(device, gpio.out_reg, out_reg_value)


def direction(handle, pin, direction):
    gpio = pin_to_gpio(pin)  
    dir_reg_value = __read_pl2303_reg(handle, gpio.dir_reg)
    dir_reg_value |= gpio.dir_mask
    __write_pl2303_reg(handle, gpio.dir_reg, dir_reg_value)

def write(handle, pin, value):
    gpio = pin_to_gpio(pin)
    out_reg_value = __read_pl2303_reg(handle, gpio.out_reg)
    if value:
        out_reg_value |= gpio.out_mask
    else:
        out_reg_value &= ~gpio.out_mask
    __write_pl2303_reg(handle, gpio.out_reg, out_reg_value)


def read(handle, pin):
    raise Exception("NOT YET IMPLEMENTED")
    #print(f"read handle: {handle}, pin {pin}")
    return 1;

==> pygpiotools-0.1.33.tar.gz: pygpiotools-0.1.33/pygpiotools/pl2303.py <==
import pygpiotools
import usb.core
import usb.util


VID = 0x67b
PID = 0x2303

class gpio:
    def __init__(self, dir_reg, dir_mask, out_reg, out_mask):
        self.dir_reg = dir_reg
        self.dir_mask = dir_mask
        self.out_reg = out_reg
        self.out_mask = out_mask

# Tested with good old PL2303HXA
__gpios = [ gpio(0x01, 0x10, 0x01, 0x40), gpio(0x01, 0x20, 0x01, 0x80)]

def pin_to_gpio(pin):
    if pin > len(__gpios):
        raise Exception(f"invalid gpio number {pin}")
    return __gpios[pin]

def create(driver, device):
    handle = pygpiotools.tty_to_usbdev(device, VID, PID)
    return handle

def __read_pl2303_reg(device, reg_num):
    buffer = device.ctrl_transfer(
        usb.util.build_request_type(usb.util.CTRL_IN, usb.util.CTRL_TYPE_VENDOR, usb.util.CTRL_RECIPIENT_DEVICE),
        0x01,
        reg_num | 0x80,
        0,
        1,
        1000)
    return buffer[0]

def __write_pl2303_reg(device, reg_num, value):
    device.ctrl_transfer(
        usb.util.build_request_type(usb.util.CTRL_OUT, usb.util.CTRL_TYPE_VENDOR, usb.util.CTRL_RECIPIENT_DEVICE),
        0x01,
        reg_num,
        value,
        None,
        1000)

def set_gpio_out(device, gpio, v):
    dir_reg_value = __read_pl2303_reg(device, gpio.dir_reg)
    dir_reg_value |= gpio.dir_mask
    __write_pl2303_reg(gpio.dir_reg, dir_reg_value)

    out_reg_value = __read_pl2303_reg(device, gpio.out_reg)
    if v:
        out_reg_value |= gpio.out_mask
    else:
        out_reg_value &= ~gpio.out_mask
    __write_pl2303_reg(device, gpio.out_reg, out_reg_value)


def direction(handle, pin, direction):
    gpio = pin_to_gpio(pin)  
    dir_reg_value = __read_pl2303_reg(handle, gpio.dir_reg)
    dir_reg_value |= gpio.dir_mask
    __write_pl2303_reg(handle, gpio.dir_reg, dir_reg_value)

def write(handle, pin, value):
    gpio = pin_to_gpio(pin)
    out_reg_value = __read_pl2303_reg(handle, gpio.out_reg)
    if value:
        out_reg_value |= gpio.out_mask
    else:
        out_reg_value &= ~gpio.out_mask
    __write_pl2303_reg(handle, gpio.out_reg, out_reg_value)


def read(handle, pin):
    raise Exception("NOT YET IMPLEMENTED")
    #print(f"read handle: {handle}, pin {pin}")
    return 1;
